/* libatasmart usage API examples
 *
 * library information:
 * - http://0pointer.de/blog/projects/being-smart.html
 * - http://git.0pointer.de/?p=libatasmart.git;a=blob;f=README
 *
 * compilation:
 * $ gcc atasmart-demo.c -latasmart -o atasmart-demo
 */

#include <stdio.h>
#include <string.h>
#include <error.h>
#include <errno.h>
#include <assert.h>
#include <unistd.h>
#include <atasmart.h>

/* routine from atasmart.c for readable output */
/* **** **** */
#define HIGHLIGHT "\x1B[1m"
#define ENDHIGHLIGHT "\x1B[0m"
char *yes_no(SkBool b) {
	return  b ? "yes" : "no";
}
char* print_name(char *s, size_t len, uint8_t id, const char *k) {
	if (k)
		strncpy(s, k, len);
	else
		snprintf(s, len, "%u", id);
	s[len-1] = 0;
	return s;
}
char *print_value(char *s, size_t len, uint64_t pretty_value, SkSmartAttributeUnit pretty_unit) {
	switch (pretty_unit) {
		case SK_SMART_ATTRIBUTE_UNIT_MSECONDS:
			if (pretty_value >= 1000LLU*60LLU*60LLU*24LLU*365LLU)
				snprintf(s, len, "%0.1f years", ((double) pretty_value)/(1000.0*60*60*24*365));
			else if (pretty_value >= 1000LLU*60LLU*60LLU*24LLU*30LLU)
				snprintf(s, len, "%0.1f months", ((double) pretty_value)/(1000.0*60*60*24*30));
			else if (pretty_value >= 1000LLU*60LLU*60LLU*24LLU)
				snprintf(s, len, "%0.1f days", ((double) pretty_value)/(1000.0*60*60*24));
			else if (pretty_value >= 1000LLU*60LLU*60LLU)
				snprintf(s, len, "%0.1f h", ((double) pretty_value)/(1000.0*60*60));
			else if (pretty_value >= 1000LLU*60LLU)
				snprintf(s, len, "%0.1f min", ((double) pretty_value)/(1000.0*60));
			else if (pretty_value >= 1000LLU)
				snprintf(s, len, "%0.1f s", ((double) pretty_value)/(1000.0));
			else
				snprintf(s, len, "%llu ms", (unsigned long long) pretty_value);
			break;
		case SK_SMART_ATTRIBUTE_UNIT_MKELVIN:
			snprintf(s, len, "%0.1f C", ((double) pretty_value - 273150) / 1000);
			break;
		case SK_SMART_ATTRIBUTE_UNIT_SECTORS:
			snprintf(s, len, "%llu sectors", (unsigned long long) pretty_value);
			break;
		case SK_SMART_ATTRIBUTE_UNIT_NONE:
			snprintf(s, len, "%llu", (unsigned long long) pretty_value);
			break;
		case SK_SMART_ATTRIBUTE_UNIT_UNKNOWN:
			snprintf(s, len, "n/a");
			break;
		case _SK_SMART_ATTRIBUTE_UNIT_MAX:
			assert(FALSE);
	}
	s[len-1] = 0;
	return s;
}
/* disk_dump_attributes callback for raw SMART data parsing */
void disk_dump_attributes(SkDisk *d, const SkSmartAttributeParsedData *a, void* userdata) {
	char name[32];
	char pretty[32];
	char tt[32], tw[32], tc[32];
	SkBool highlight;
	
	snprintf(tt, sizeof(tt), "%3u", a->threshold);
	tt[sizeof(tt)-1] = 0;
	snprintf(tw, sizeof(tw), "%3u", a->worst_value);
	tw[sizeof(tw)-1] = 0;
	snprintf(tc, sizeof(tc), "%3u", a->current_value);
	tc[sizeof(tc)-1] = 0;
	
	highlight = a->warn && isatty(1);
	
	if (highlight)
		fprintf(stderr, HIGHLIGHT);
	
	printf("%3u %-27s %-3s   %-3s   %-3s   %-11s 0x%02x%02x%02x%02x%02x%02x %-7s %-7s %-4s %-4s\n",
		a->id,
		print_name(name, sizeof(name), a->id, a->name),
		a->current_value_valid ? tc : "n/a",
		a->worst_value_valid ? tw : "n/a",
		a->threshold_valid ? tt : "n/a",
		print_value(pretty, sizeof(pretty), a->pretty_value, a->pretty_unit),
		a->raw[0], a->raw[1], a->raw[2], a->raw[3], a->raw[4], a->raw[5],
		a->prefailure ? "prefail" : "old-age",
		a->online ? "online" : "offline",
		a->good_now_valid ? yes_no(a->good_now) : "n/a",
		a->good_in_the_past_valid ? yes_no(a->good_in_the_past) : "n/a"
	);
	if (highlight)
		fprintf(stderr, ENDHIGHLIGHT);
}
/* **** **** */

int main(int argc, char const* argv[])
{
	char *name = "/dev/sda";
	SkDisk *disk;
	
	int r = 0;
	
	/* open disk by name and init it */
	if ((r = sk_disk_open(name, &disk)) < 0) {
		fprintf(stderr, "Failed to open disk %s: %s\n", name, strerror(errno));
		return -1;
	}
	
	/* getting raw smart data */
	if ((r = sk_disk_smart_read_data(disk)) < 0) {
		fprintf(stderr, "Failed to read SMART data: %s\n", strerror(errno));
		goto exit;
	}
	
	/* getting temperature in Kelvin */
	uint64_t t;
	if ((r = sk_disk_smart_get_temperature(disk, &t)) < 0) {
		fprintf(stderr, "Failed to get temperature: %s\n", strerror(errno));
		goto exit;
	}
	
	printf("t = %llu\n", (unsigned long long) t);
	
	/* getting serial number using appropriate struct */
	const SkIdentifyParsedData *data;
	if ((r = sk_disk_identify_parse(disk, &data)) < 0) {
		fprintf(stderr, "Failed to identify parsed data: %s\n", strerror(errno));
		goto exit;
	}
	
	printf("s = %s\n", data->serial);
	
	sk_disk_smart_parse_attributes(disk, disk_dump_attributes, NULL);
	
	/* sk_disk_dump(disk); */
	
	return 0;
	
exit:
	if (disk) {
		printf("Cleaning memory...\n");
		sk_disk_free(disk);
	}
	return r;
}
