/* Create a file with one primary partition (ext3) in it, an extended partition
 * with 2 partitions , the first one in reiserfs and the second one with no 
 * filesystem. That second being the bigger one. */

/* to be compiled with:
 * gcc -l parted -o create create.c 
 */

/* Copyright (C) 2007 Stephane Pasco
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <stdio.h>
#include <parted/parted.h>

const int SIZE = 256; /* in MegaBytes */
const int PART1_SIZE = 40; /* in MegaBytes */
const int PART2_SIZE = 40; /* in MegaBytes */
const char *FILENAME = "/home/tester/Desktop/vdisk";

void print_PedDevice(PedDevice *p)
{
	printf("path: %s\n",p->path);
	printf("model: %s\n",p->model);
	printf("open_count: %d\n",p->open_count);
	printf("phys_sector_size: %d\n",p->phys_sector_size);
	printf("sector_size: %d\n",p->sector_size);
	printf("length: %lli\n",p->length);
	printf("cylinders: %d\n",p->hw_geom.cylinders);
	printf("heads: %d\n",p->hw_geom.heads);
	printf("sectors: %d\n",p->hw_geom.sectors);
}

void print_PedGeometry(PedGeometry *g)
{
	printf("start sector: %lli\n",g->start);
	printf("end sector: %lli\n",g->end);
	printf("length: %lli\n",g->length);
}

/* create an empty file (filled with 0) of size MB */
void create_empty_file(const char *filename,int size)
{
	FILE *fd;
	int i;
	const void *onekb;

	fd = fopen(filename,"w");
	onekb=calloc(1024,1);

	for(i=0;i<size*1024;i++) {
		fwrite(onekb,1024,1,fd);
	}	
	fclose(fd);
}

int main(void)
{
	int ret,size;
	long long start,end;

	PedDevice *device;
	PedDisk *disk;
	PedPartition *part,*ext;
	PedDiskType *pttype;
	PedFileSystemType *fs_type;
	PedConstraint *constraint;
	PedGeometry *geom;

	/* create an empty file for our test */
	create_empty_file(FILENAME,SIZE);

	device = ped_device_get(FILENAME);
	print_PedDevice(device);
	
	pttype = ped_disk_type_get("msdos");
	/* create a new partition table */
	disk = ped_disk_new_fresh(device,pttype);
	
	/* Create first primary partition */
	size = PART1_SIZE;
	start = 1;
	end = (long long)(size*1024*1024/device->sector_size);
	fs_type = ped_file_system_type_get("ext2");
	part = ped_partition_new(disk,PED_PARTITION_NORMAL,fs_type,start,end);

	ped_disk_add_partition(disk,part,ped_constraint_exact(&part->geom));
	ped_file_system_create(&part->geom,fs_type,NULL);
	
	ped_disk_commit_to_dev(disk);

	/* create the extended partition */
	start = end + 1;
	end = device->length - 1; /* all the space left */
	fs_type = ped_file_system_type_get("extended");
	ext = ped_partition_new(disk,PED_PARTITION_EXTENDED,fs_type,start,end);

	ped_disk_add_partition(disk,ext,ped_constraint_exact(&ext->geom));
	
	ped_disk_commit_to_dev(disk);

	/* create first logical partition */
	size = PART2_SIZE;
	start = ext->geom.start + 1;
	end = start + (long long)(size*1024*1024/device->sector_size);
	fs_type = ped_file_system_type_get("ext2");
	part = ped_partition_new(disk,PED_PARTITION_LOGICAL,fs_type,start,end);

	ped_disk_add_partition(disk,part,ped_constraint_exact(&part->geom));
	ped_file_system_create(&part->geom,fs_type,NULL);

	ped_disk_commit_to_dev(disk);

	/* create last logical partition */
	start = end + 1;
	end = ext->geom.end; /* all the space left */
	fs_type = NULL;
	part = ped_partition_new(disk,PED_PARTITION_LOGICAL,fs_type,start,end);
	constraint = ped_constraint_new_from_max(&part->geom);

	ped_disk_add_partition(disk,part,constraint);

	ped_disk_commit_to_dev(disk);

	ped_disk_print(disk);

	ped_disk_destroy(disk);
}

